<?php

namespace Drupal\k8s\Plugin\Block;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\k8s\Entity\K8sNamespace;
use Drupal\k8s\Entity\K8sNode;

/**
 * Provides a block of the costs.
 *
 * @Block(
 *   id = "k8s_namespace_costs",
 *   admin_label = @Translation("K8s Namespace Costs"),
 *   category = @Translation("K8s")
 * )
 */
class K8sNamespaceCostsBlock extends K8sBaseBlock {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'display_view_k8s_namespace_list_only' => 1,
      'aws_cloud_ec2_cost_type' => 'ri_one_year',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {

    $form['display_view_k8s_namespace_list_only'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display this block in K8s Namespace list page only'),
      '#default_value' => $this->configuration['display_view_k8s_namespace_list_only'],
    ];

    $form['aws_cloud_ec2_cost_type'] = [
      '#type' => 'select',
      '#title' => $this->t('AWS Cloud EC2 Cost Type'),
      '#options' => $this->getEc2CostTypes(),
      '#default_value' => $this->configuration['aws_cloud_ec2_cost_type'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {

    $this->configuration['display_view_k8s_namespace_list_only']
      = $form_state->getValue('display_view_k8s_namespace_list_only');

    $this->configuration['aws_cloud_ec2_cost_type']
      = $form_state->getValue('aws_cloud_ec2_cost_type');
  }

  /**
   * {@inheritdoc}
   */
  public function build() {

    // If aws_cloud module is not enabled; do nothing.
    if (!$this->moduleHandler->moduleExists('aws_cloud')) {
      return [
        '#type' => 'markup',
        '#markup' => $this->t('This block requires the AWS Cloud
        module to be enabled.'),
      ];
    }

    if ($this->isK8sEmpty() === TRUE) {
      return [];
    }

    // If display_view_k8s_namespace_list_only is checked, do nothing.
    $route_name = $this->routeMatch->getRouteName();
    if ($route_name !== 'view.k8s_namespace.list'
      && $this->configuration['display_view_k8s_namespace_list_only']) {
      return [];
    }

    $nodes = [];
    $namespaces = [];

    $cloud_context = $this->routeMatch->getParameter('cloud_context');
    // If cloud_context is passed, filter entities with it.
    if (!empty($cloud_context)) {
      $nodes = $this->entityTypeManager
        ->getStorage('k8s_node')->loadByProperties(
          [
            'cloud_context' => $cloud_context,
          ]
        );

      $namespaces = $this->entityTypeManager
        ->getStorage('k8s_namespace')->loadByProperties(
          [
            'cloud_context' => $cloud_context,
          ]
        );

      // If nodes or namespaces are empty, ask the user
      // to try updating the cloud_context to bring in
      // the latest data.
      if (empty($nodes) || empty($namespaces)) {
        $message = $this->t('Nodes and/or namespaces not found for K8s Namespace Cost block.');
        $this->setUpdateMessage($cloud_context, $message);
      }
    }
    else {
      // Load all nodes and namespaces.
      $node_ids = $this->entityTypeManager
        ->getStorage('k8s_node')->getQuery()
        ->execute();
      foreach ($node_ids ?: [] as $key => $node) {
        $nodes[] = K8sNode::load($key);
      }
      $namespace_ids = $this->entityTypeManager
        ->getStorage('k8s_namespace')->getQuery()
        ->execute();
      foreach ($namespace_ids ?: [] as $key => $namespace) {
        $namespaces[] = K8sNamespace::load($key);
      }
    }

    $build = [];

    $total_costs = $this->getTotalCosts($nodes);

    // $cpu_capacity = $this->getCpuCapacity($nodes);
    $cpu_capacity = array_sum(array_map(static function ($node) {
      return $node->getCpuCapacity();
    }, $nodes));

    $memory_capacity = array_sum(array_map(static function ($node) {
      return $node->getMemoryCapacity();
    }, $nodes));

    $pod_capacity = array_sum(array_map(static function ($node) {
      return $node->getPodsCapacity();
    }, $nodes));

    // Get row data.
    $rows = [];
    foreach ($namespaces ?: [] as $namespace) {
      $pods = $this->entityTypeManager
        ->getStorage('k8s_pod')->loadByProperties([
          'cloud_context' => $namespace->getCloudContext(),
          'namespace' => $namespace->getName(),
        ]
      );

      $row['namespace'] = [
        'data' => [
          '#type' => 'link',
          '#title' => $namespace->getName(),
          '#url' => Url::fromRoute(
            'entity.k8s_namespace.canonical',
            [
              'cloud_context' => $namespace->getCloudContext(),
              'k8s_namespace' => $namespace->id(),
            ]
          ),
        ],
      ];

      $cpu_usage = array_sum(array_map(static function ($pod) {
        return $pod->getCpuUsage();
      }, $pods));

      $memory_usage = array_sum(array_map(static function ($pod) {
        return $pod->getMemoryUsage();
      }, $pods));

      $pod_usage = count($pods);

      $costs = $total_costs > 0 ? ($cpu_usage / $cpu_capacity + $memory_usage / $memory_capacity + $pod_usage / $pod_capacity) / 3 * $total_costs : 0;

      $row['costs'] = $total_costs > 0 ? $this->formatCosts($costs, $total_costs) : 0;
      $row['cpu_usage'] = $cpu_capacity > 0 ? $this->formatCpuUsage($cpu_usage, $cpu_capacity) : 0;
      $row['memory_usage'] = $memory_capacity > 0 ? $this->formatMemoryUsage($memory_usage, $memory_capacity) : 0;
      $row['pod_usage'] = $pod_capacity > 0 ? $this->formatPodUsage($pod_usage, $pod_capacity) : 0;

      $rows[] = $row;
    }

    $table = [
      '#theme' => 'table',
      '#header' => [
        $this->t('Namespace'),
        $this->t('Total Costs ($)'),
        $this->t('CPU (Usage)'),
        $this->t('Memory (Usage)'),
        $this->t('Pods Allocation'),
      ],
      '#rows' => $rows,
    ];

    $cost_types = $this->getEc2CostTypes();
    $cost_type = $this->configuration['aws_cloud_ec2_cost_type'];
    $build['k8s_namespace_costs'] = [
      '#type' => 'details',
      '#title' => $this->t('Namespace Costs') . " ($cost_types[$cost_type])",
      '#open' => TRUE,
    ];

    $build['k8s_namespace_costs'][] = $table;

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheMaxAge() {
    $this->killSwitch->trigger();
    // BigPipe/#cache/max-age is breaking my block javascript
    // https://www.drupal.org/forum/support/module-development-and-code-questions/2016-07-17/bigpipecachemax-age-is-breaking-my
    // "a slight delay of a second or two before the charts are built.
    // That seems to work, though it is a janky solution.".
    return $this->moduleHandler->moduleExists('big_pipe') ? 1 : 0;
  }

  /**
   * Format costs.
   *
   * @param int $costs
   *   The costs.
   * @param int $total_costs
   *   The total costs.
   *
   * @return string
   *   The formatted costs string.
   */
  private function formatCosts($costs, $total_costs) {
    $costs_str = round($costs, 2);
    $percentage = round($costs / $total_costs * 100, 2);
    return "$costs_str ($percentage%)";
  }

  /**
   * Format cpu usage.
   *
   * @param float $cpu_usage
   *   The cpu usage.
   * @param float $cpu_capacity
   *   The cpu capacity.
   *
   * @return string
   *   The formatted cpu usage string.
   */
  private function formatCpuUsage($cpu_usage, $cpu_capacity) {
    $cpu_str = round($cpu_usage, 2);
    $percentage = round($cpu_usage / $cpu_capacity * 100, 2);
    return "$cpu_str ($percentage%)";
  }

  /**
   * Format memory usage.
   *
   * @param int $memory_usage
   *   The memory usage.
   * @param int $memory_capacity
   *   The memory capacity.
   *
   * @return string
   *   The formatted memory usage string.
   */
  private function formatMemoryUsage($memory_usage, $memory_capacity) {
    $memory_str = k8s_format_memory($memory_usage);
    $percentage = round($memory_usage / $memory_capacity * 100, 2);
    return "$memory_str ($percentage%)";
  }

  /**
   * Format pod usage.
   *
   * @param int $pod_usage
   *   The pod usage.
   * @param int $pod_capacity
   *   The pod capacity.
   *
   * @return string
   *   The formatted pod usage string.
   */
  private function formatPodUsage($pod_usage, $pod_capacity) {
    $percentage = round($pod_usage / $pod_capacity * 100, 2);
    return "$pod_usage/$pod_capacity ($percentage%)";
  }

  /**
   * Get total costs of nodes.
   *
   * @param array $nodes
   *   The k8s_node entities.
   *
   * @return int
   *   The total costs of nodes.
   */
  private function getTotalCosts(array $nodes) {
    $costs = 0;

    if (!$this->moduleHandler->moduleExists('aws_cloud')) {
      return $costs;
    }

    /* @var \Drupal\aws_cloud\Service\Pricing\InstanceTypePriceDataProvider $price_data_provider */
    $price_date_provider = \Drupal::service('aws_cloud.instance_type_price_data_provider');

    foreach ($nodes ?: [] as $node) {
      // Get instance type and region.
      $region = NULL;
      $instance_type = NULL;
      $labels = $node->get('labels');
      foreach ($labels ?: [] as $item) {
        if ($item->getItemKey() === 'beta.kubernetes.io/instance-type') {
          $instance_type = $item->getItemValue();
        }
        elseif ($item->getItemKey() === 'failure-domain.beta.kubernetes.io/region') {
          $region = $item->getItemValue();
        }
      }

      if (empty($instance_type) || empty($region)) {
        continue;
      }

      $price_data = $price_date_provider->getDataByRegion($region);
      foreach ($price_data ?: [] as $item) {
        if ($item['instance_type'] === $instance_type) {
          $cost_type = $this->configuration['aws_cloud_ec2_cost_type'];
          if (!empty($item[$cost_type])) {
            if ($cost_type === 'ri_three_year') {
              $costs += $item[$cost_type] / 3;
            }
            else {
              $costs += $item[$cost_type];
            }
          }
          break;
        }
      }
    }

    return $costs;
  }

  /**
   * Get a list of Ec2 cost Types such as On-demand, RI one or three years.
   *
   * @return array
   *   The list of Ec2 cost Types.
   */
  private function getEc2CostTypes() {
    return [
      'on_demand_yearly'  => $this->t('On-demand Yearly'),
      'ri_one_year'       => $this->t('RI 1 Year'),
      'ri_three_year'     => $this->t('RI 3 Years'),
    ];
  }

}
