<?php

namespace Drupal\k8s\Service\K8sClientExtension;

use BadMethodCallException;
use Drupal\cloud\Traits\CloudContentEntityTrait;

use Drupal\k8s\Service\K8sClientExtension\Repositories\K8sApiServiceRepository;
use Drupal\k8s\Service\K8sClientExtension\Repositories\K8sClusterRoleBindingRepository;
use Drupal\k8s\Service\K8sClientExtension\Repositories\K8sClusterRoleRepository;
use Drupal\k8s\Service\K8sClientExtension\Repositories\K8sLimitRangeRepository;
use Drupal\k8s\Service\K8sClientExtension\Repositories\K8sMetricsNodeRepository;
use Drupal\k8s\Service\K8sClientExtension\Repositories\K8sMetricsPodRepository;
use Drupal\k8s\Service\K8sClientExtension\Repositories\K8sQuotaRepository;
use Drupal\k8s\Service\K8sClientExtension\Repositories\K8sRoleBindingRepository;
use Drupal\k8s\Service\K8sClientExtension\Repositories\K8sRoleRepository;
use Drupal\k8s\Service\K8sClientExtension\Repositories\K8sServiceAccountRepository;
use Drupal\k8s\Service\K8sClientExtension\Repositories\K8sStatefulSetRepository;
use Drupal\k8s\Service\K8sClientExtension\Repositories\K8sStorageClassRepository;
use Drupal\k8s\Service\K8sServiceException;
use GuzzleHttp\Client as GuzzleClient;
use Maclof\Kubernetes\Client;
use Maclof\Kubernetes\Exceptions\BadRequestException;

/**
 * K8s client.
 */
class K8sClient extends Client {

  use CloudContentEntityTrait;

  /**
   * The constructor.
   *
   * @param array $options
   *   Options.
   * @param \GuzzleHttp\Client $guzzle_client
   *   The guzzle client.
   */
  public function __construct(
    array $options = [],
    GuzzleClient $guzzle_client = NULL
  ) {
    parent::__construct($options, $guzzle_client);

    // Setup the messenger object.
    $this->messenger();

    // Change the repository of quotas.
    $this->classMap['quotas'] = K8sQuotaRepository::class;

    // Change the repository of limit ranges.
    $this->classMap['limitRanges'] = K8sLimitRangeRepository::class;

    // Add the repository of metrics pods.
    $this->classMap['metricsPods'] = K8sMetricsPodRepository::class;

    // Add the repository of metrics nodes.
    $this->classMap['metricsNodes'] = K8sMetricsNodeRepository::class;

    // Add the repository of roles.
    $this->classMap['roles'] = K8sRoleRepository::class;

    // Add the repository of role bindings.
    $this->classMap['roleBindings'] = K8sRoleBindingRepository::class;

    // Add the repository of cluster roles.
    $this->classMap['clusterRoles'] = K8sClusterRoleRepository::class;

    // Add the repository of cluster role bindings.
    $this->classMap['clusterRoleBindings'] = K8sClusterRoleBindingRepository::class;

    // Add the repository of storage classes.
    $this->classMap['storageClasses'] = K8sStorageClassRepository::class;

    // Add the repository of stateful sets.
    $this->classMap['statefulSets'] = K8sStatefulSetRepository::class;

    // Add the repository of api services.
    $this->classMap['apiServices'] = K8sApiServiceRepository::class;

    // Add the repository of service accounts.
    $this->classMap['serviceAccounts'] = K8sServiceAccountRepository::class;

  }

  /**
   * {@inheritdoc}
   */
  public function sendRequest(
    $method,
    $uri,
    $query = [],
    $body = [],
    $namespace = TRUE,
    $apiVersion = NULL
  ) {
    try {
      if ($method === 'PUT' || $method === 'POST') {
        // If the resource type is cluster role, the apiGroups maybe empty,
        // and it can't be removed.
        if ($uri !== '/clusterroles') {
          $array = $this->removeEmptyProperties(json_decode($body, TRUE));
          $body = json_encode($array, JSON_PRETTY_PRINT);
        }
      }

      if (empty($this->namespace)) {
        $namespace = FALSE;
      }

      return parent::sendRequest(
        $method,
        $uri,
        $query,
        $body,
        $namespace,
        $apiVersion
      );
    }
    catch (BadRequestException $e) {
      $error_info = json_decode($e->getMessage(), TRUE);
      if (empty($error_info)) {
        throw new K8sServiceException($this->t(
          'Unknown error occurred when calling K8s API.'
        ));
      }

      $this->messenger->addError($this->t(
        'An error occurred when calling K8s API: @method @uri',
        [
          '@method' => $method,
          '@uri' => $uri,
        ]
      ));

      $this->messenger->addError($this->t(
        'Status Code: @status_code', ['@status_code' => $error_info['code']]
      ));

      $this->messenger->addError($this->t(
        'Error reason: @error_reason', ['@error_reason' => $error_info['reason']]
      ));

      $this->messenger->addError($this->t(
        'Message: @msg', ['@msg' => $error_info['message']]
      ));

      throw new K8sServiceException($this->t(
        'An error occurred when calling K8s API.'
      ));
    }
    catch (\Exception $e) {
      throw new K8sServiceException($e->getMessage());
    }
  }

  /**
   * {@inheritdoc}
   */
  public function __call($name, array $args) {
    if (isset($this->classMap[$name])) {
      $class = 'Maclof\Kubernetes\\' . $this->classMap[$name];

      if (in_array($name, [
        'quotas',
        'metricsPods',
        'metricsNodes',
        'limitRanges',
        'roles',
        'roleBindings',
        'clusterRoles',
        'clusterRoleBindings',
        'storageClasses',
        'statefulSets',
        'apiServices',
        'serviceAccounts',
      ])) {
        $class = $this->classMap[$name];
      }

      return $this->classInstances[$name] ?? new $class($this);
    }

    throw new BadMethodCallException('No client methods exist with the name: ' . $name);
  }

  /**
   * Remove empty properties.
   *
   * @param array $haystack
   *   The array.
   *
   * @return array
   *   The array whose empty properties were removed.
   */
  private function removeEmptyProperties(array $haystack) {
    foreach ($haystack ?: [] as $key => $value) {
      if (is_array($value)) {
        $haystack[$key] = $this->removeEmptyProperties($value);
      }

      if (empty($haystack[$key])) {
        unset($haystack[$key]);
      }
    }

    return $haystack;
  }

}
